export declare type ObjectLiteral<T = unknown> = Record<string, T>;
export declare type ObjectInstance<T = unknown> = {
    constructor: Function;
    [key: string]: unknown;
};
export declare type Primitive = string | number | bigint | boolean | symbol | null | undefined;
declare function isNullOrUndefined(v: unknown): boolean;
declare function isObject(o: unknown): o is ObjectInstance | ObjectLiteral;
declare function isObjectLiteral(o: unknown): o is ObjectLiteral;
declare function isEmptyObjectLiteral(o: unknown): o is {};
declare function isObjectInstance(o: unknown): o is ObjectInstance;
declare function isArray(a: unknown): a is unknown[];
declare function isEmptyArray(a: unknown): a is [];
declare function isArrayOfObjects(a: unknown): a is (ObjectInstance | ObjectLiteral)[];
declare function isArrayOfObjectLiterals(a: unknown): a is ObjectLiteral[];
declare function isArrayOfPrimitives(a: unknown): a is Primitive[];
declare function isArrayOfType<T extends string | number | boolean>(a: unknown, type: T extends string ? 'string' : T extends number ? 'number' : 'boolean'): a is Array<T>;
declare function isArrayWhereEvery<T>(a: unknown, fun: (value: unknown) => boolean): a is Array<T>;
declare function isObjectLiteralWhereEvery<T>(o: unknown, fun: (value: unknown) => boolean): o is ObjectLiteral<T>;
declare function areValuesEqual(a: unknown, b: unknown): boolean;
export declare enum ComparisonResult {
    Equal = "EQUAL",
    NotEqual = "NOT_EQUAL",
    DefaultComparison = "DEFAULT_COMPARISON"
}
declare type AreEqualOptions = {
    compare: (a: any, b: any) => ComparisonResult;
};
declare function areObjectsEqual(a: ObjectLiteral | ObjectInstance, b: ObjectLiteral | ObjectInstance, options?: AreEqualOptions): boolean;
declare function areArraysEqual(a: unknown[], b: unknown[], options?: AreEqualOptions): boolean;
declare function hasProperty(o: ObjectInstance | ObjectLiteral, prop: string): boolean;
declare function hasProperties(o: ObjectInstance | ObjectLiteral, props: string[]): boolean;
declare function filterProperties<T = unknown>(o: object, arg: string[] | ((key: string, val: T) => boolean)): {};
declare function rejectProperties<T = unknown>(o: object, arg: string[] | ((key: string, val: T) => boolean)): {
    [k: string]: any;
};
declare function takeProperties<T = unknown>(o: object, arg: string[] | ((key: string, val: T) => boolean)): {
    filtered: {};
    rejected: {};
    undefined: {};
};
declare function removeArrayElements<T = unknown>(array: T[], listOfValues: T[]): T[];
declare function removeArrayElement<T = unknown>(array: T[], valueOrFun: T | ((value: T) => boolean)): T[];
declare function removeArrayElementByIndex<T = unknown>(array: T[], index: number): T[];
declare function differenceArraysOfPrimitives<T extends Primitive>(a1: T[], a2: T[]): T[];
declare function isObjectSubset(superObject: ObjectInstance | ObjectLiteral, subObject: ObjectInstance | ObjectLiteral): boolean;
declare function isArraySubset(superArray: unknown[], subArray: unknown[]): boolean;
declare function isPrimitive(value: unknown): boolean;
declare function cloneShape<T = unknown>(value: T): T;
declare function deepFreeze(o: unknown[] | ObjectInstance | ObjectLiteral): Readonly<ObjectInstance<unknown> | ObjectLiteral<unknown> | unknown[]>;
declare function sortProperties(o: ObjectLiteral): {
    [k: string]: unknown;
};
declare type RangeOptionsWithCount = {
    start?: number;
    count: number;
    endInclusive?: never;
    endExclusive?: never;
};
declare type RangeOptionsWithEndInclusive = {
    start?: number;
    count?: never;
    endInclusive: number;
    endExclusive?: never;
};
declare type RangeOptionsWithEndExclusive = {
    start?: number;
    count?: never;
    endInclusive?: never;
    endExclusive: number;
};
declare type RangeOptions = RangeOptionsWithCount | RangeOptionsWithEndInclusive | RangeOptionsWithEndExclusive;
declare function range(options: RangeOptions): number[];
declare function duplicate<T = unknown>(value: T, count: number, transformFun?: (v: T, _i: number) => T): T[];
declare function unique<T>(a: T[], fun?: (val: T) => unknown): T[];
export { areArraysEqual, areObjectsEqual, areValuesEqual, cloneShape, deepFreeze, differenceArraysOfPrimitives, duplicate, filterProperties, hasProperty, hasProperties, isArray, isArrayOfObjects, isArrayOfObjectLiterals, isArrayOfPrimitives, isArrayOfType, isArraySubset, isArrayWhereEvery, isEmptyArray, isEmptyObjectLiteral, isNullOrUndefined, isObject, isObjectInstance, isObjectLiteral, isObjectLiteralWhereEvery, isObjectSubset, isPrimitive, range, rejectProperties, removeArrayElement, removeArrayElementByIndex, removeArrayElements, sortProperties, takeProperties, unique };
